package ingest

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"io/ioutil"
	"log"
	"net/http"
	"strconv"
)

type RequestPayload struct {
	CachePath string `json:"cache_path"`
	CalSpl    string `json:"cal_spl"`
}

type ResponsePayload struct {
	WorkflowName      string `json:"workflow_name"`
	WorkflowRequestId int    `json:"workflow_request_id"`
}

//CallIngestionEndpoint
/**
 * Call the SECI ingestion endpoint to initiate the workflow
 *
 * @param img_set_path A string with the path to a VLASS image set
 * @param locator A string with the locator for the image set
 * @param capo_profile A string with the path to the CAPO property file used to
 * get the Workflow service URL
 **/
func CallIngestionEndpoint(imgSetPath string, locator string, input CapoInput) {
	settings := getCapoConfig(input).SettingsForPrefix("edu.nrao.workspaces.WorkflowSettings")
	workflowUrl := settings.GetString("serviceUrl")

	payload := RequestPayload{
		CachePath: imgSetPath,
		CalSpl:    locator,
	}

	// Convert payload to json
	payloadJson, err := json.Marshal(payload)
	checkError(err)

	fmt.Println(string(payloadJson))

	// Get post request
	postUrl := workflowUrl + "/workflows/requests/vlass_ingest/seci"
	resp, err := http.Post(postUrl, "application/json", bytes.NewBuffer(payloadJson))
	checkError(err)

	defer func(Body io.ReadCloser) {
		err := Body.Close()
		checkError(err)
	}(resp.Body)

	if resp.StatusCode != http.StatusOK {
		panic(fmt.Sprintf("Error making POST request to URL %s, got status code %d", postUrl, resp.StatusCode))
	}

	var respBody ResponsePayload
	respBytes, err := ioutil.ReadAll(resp.Body)
	checkError(err)
	err = json.Unmarshal(respBytes, &respBody)
	checkError(err)

	log.Println("POST Request Made")
	log.Printf("%s workflow created.\n", respBody.WorkflowName)
	log.Println("Workflow Request Id: ", respBody.WorkflowRequestId)
	log.Printf("\nTo check ingestion status, run: seci_ingestion_status -id %s", strconv.Itoa(respBody.WorkflowRequestId))
}
